import logging
import json

from .api import SHCAPI

logger = logging.getLogger("salusshcpy")


class SALUSDevice:
    def __init__(self, api: SHCAPI, raw_device):
        self._api = api
        self._raw_device = raw_device
        self._callbacks = {}

    @property
    def is_available(self):
        return True if self._raw_device.get("sZDOInfo", {}).get("OnlineStatus_i", 1) == 1 else False

    @property
    def name(self):
        return json.loads(self.s_zdo.get("DeviceName", '{"deviceName": "Unknown"}'))\
            .get("deviceName", None)

    @property
    def unique_id(self):
        return self.data.get("UniID", None)

    @property
    def supported_features(self):
        return None

    @property
    def device_class(self):
        return ""

    @property
    def data(self):
        return self._raw_device.get("data", None)

    @property
    def sit_600th(self):
        return self._raw_device.get("sIT600TH", {})

    @property
    def device_l(self):
        return self._raw_device.get("DeviceL", {})

    @property
    def s_zdo(self):
        return self._raw_device.get("sZDO", {})

    @property
    def model(self):
        return self.device_l.get("ModelIdentifier_i", None)

    @property
    def device_type(self):
        return self.data.get("DeviceType", None)

    def summary(self):
        print(f"Device: {self.unique_id}")
        print(f"  Name          : {self.name}")
        print(f"  Manufacturer  : Salus")
        print(f"  Unique ID        : {self.unique_id}")

    def process_long_polling_poll_result(self, raw_result):
        self._raw_device = raw_result

        for callback in self._callbacks:
            self._callbacks[callback]()

    def subscribe_callback(self, entity, callback):
        self._callbacks[entity] = callback

    def unsubscribe_callback(self, entity):
        self._callbacks.pop(entity, None)

    def update(self):
        return None



import hashlib

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import padding
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes


class IT600Encryptor:
    iv = bytes([0x88, 0xa6, 0xb0, 0x79, 0x5d, 0x85, 0xdb, 0xfc, 0xe6, 0xe0, 0xb3, 0xe9, 0xa6, 0x29, 0x65, 0x4b])

    def __init__(self, euid: str):
        key: bytes = hashlib.md5(f"Salus-{euid.lower()}".encode("utf-8")).digest() + bytes([0] * 16)
        self.cipher = Cipher(algorithms.AES(key), modes.CBC(self.iv), default_backend())

    def encrypt(self, plain: str) -> bytes:
        encryptor = self.cipher.encryptor()
        padder = padding.PKCS7(128).padder()
        padded_data: bytes = padder.update(plain.encode("utf-8")) + padder.finalize()
        return encryptor.update(padded_data) + encryptor.finalize()

    def decrypt(self, cypher: bytes) -> str:
        decryptor = self.cipher.decryptor()
        padded_data: bytes = decryptor.update(cypher) + decryptor.finalize()
        unpadder = padding.PKCS7(128).unpadder()
        plain: bytes = unpadder.update(padded_data) + unpadder.finalize()
        return plain.decode("utf-8")



import logging
import typing

from .device import SALUSDevice

from .models_impl import (
    SALUSThermostat,
    SUPPORTED_MODELS,
    build
)

logger = logging.getLogger("salusshcpy")


class SALUSDeviceHelper:
    def __init__(self, api):
        self._api = api
        self._devices_by_model = {}
        for model in SUPPORTED_MODELS:
            self._devices_by_model[model] = {}

    def device_init(self, raw_device):
        device_id = raw_device["data"]["UniID"]
        device_model = raw_device["DeviceL"]["ModelIdentifier_i"]
        print(device_model)
        if device_model in SUPPORTED_MODELS:
            device = build(api=self._api, raw_device=raw_device)
            self._devices_by_model[device_model][device_id] = device
        else:
            device = SALUSDevice(api=self._api, raw_device=raw_device)
        return device

    @property
    def thermostats(self) -> typing.Sequence[SALUSThermostat]:
        if "Thermostat" not in SUPPORTED_MODELS:
            return []
        return list(self._devices_by_model["Thermostat"].values())




import typing
import threading
import time
import logging

from .api import SHCAPI, JSONRPCError
from .device import SALUSDevice
from .information import SALUSInformation
from .device_helper import SALUSDeviceHelper

logger = logging.getLogger("salusshcpy")


class SALUSSession:

    def __init__(self, controller_ip: str, euid: str):
        self._api = SHCAPI(controller_ip=controller_ip, euid=euid)
        self._device_helper = SALUSDeviceHelper(self._api)
        self._information = None
        self._devices_by_id = {}

        self._get_information()

        self._polling_thread = None
        self._stop_polling_thread = False

        self.reset_connection_listener = None

    def _get_information(self):
        raw_information = self._api.get_information()
        self._information = SALUSInformation(api=self.api, raw_information=raw_information)

        for raw_device in self._information.other_devices_raw:
            device_id = raw_device["data"]["UniID"]
            self._devices_by_id[device_id] = self._device_helper.device_init(raw_device)

    def _long_poll(self, wait_seconds=0):
        try:
            raw_results = self.api.long_polling_poll(wait_seconds)
            for raw_result in list(
                    filter(
                        lambda x: not x["data"]["DeviceType"] == 200 and not x["data"]["DeviceType"] == 300,
                        raw_results
                    )
            ):
                self._process_long_polling_poll_result(raw_result)

            return True
        except JSONRPCError as json_rpc_error:
            raise json_rpc_error

    def _process_long_polling_poll_result(self, raw_result):
        device_id = raw_result["data"]["UniID"]
        if device_id in self._devices_by_id.keys():
            device = self._devices_by_id[device_id]
            device.process_long_polling_poll_result(raw_result)
        else:
            logger.debug(f"Skipping polling result with unknown device id {device_id}.")
            print(f"Skipping polling result with unknown device id {device_id}.")
            return

    def start_polling(self):
        if self._polling_thread is None:

            def polling_thread_main():
                while not self._stop_polling_thread:
                    try:
                        if not self._long_poll():
                            logging.warning("_long_poll returned False. Waiting 1 second.")
                            time.sleep(1.0)
                    except RuntimeError as err:
                        self._stop_polling_thread = True
                        logging.info( "Stopping polling thread after expected runtime error.")
                        logging.info(f"Error description: {err}. {err.args}")
                    except Exception as ex:
                        logging.error(f"Error in polling thread: {ex}. Waiting 15 seconds.")
                        time.sleep(15.0)

            self._polling_thread = threading.Thread(target=polling_thread_main, name="SALUSPollingThread")
            self._polling_thread.start()

        else:
            raise ValueError("Already polling!")

    @property
    def devices(self) -> typing.Sequence[SALUSDevice]:
        return list(self._devices_by_id.values())

    def device(self, device_id) -> SALUSDevice:
        return self._devices_by_id[device_id]

    @property
    def information(self) -> SALUSInformation:
        return self._information

    @property
    def api(self):
        return self._api

